@@ -279,6 +279,19 @@ struct __dev_attr<::cudaDevAttrNumaConfig> //
 };
 #  endif // _CCCL_CTK_AT_LEAST(12, 2)
 
+#  if _CCCL_CTK_AT_LEAST(12, 6)
+template <>
+struct __dev_attr<::cudaDevAttrHostNumaMemoryPoolsSupported>
+    : __dev_attr_impl<::cudaDevAttrHostNumaMemoryPoolsSupported, bool>
+{};
+#  endif // ^^^ _CCCL_CTK_AT_LEAST(12, 6) ^^^
+
+#  if _CCCL_CTK_AT_LEAST(13, 0)
+template <>
+struct __dev_attr<::cudaDevAttrHostMemoryPoolsSupported> : __dev_attr_impl<::cudaDevAttrHostMemoryPoolsSupported, bool>
+{};
+#  endif // ^^^ _CCCL_CTK_AT_LEAST(13, 0) ^^^
+
 namespace device_attributes
 {
 // Maximum number of threads per block
@@ -736,6 +749,16 @@ using numa_id_t = __dev_attr<::cudaDevAttrNumaId>;
 static constexpr numa_id_t numa_id{};
 #  endif // _CCCL_CTK_AT_LEAST(12, 2)
 
+#  if _CCCL_CTK_AT_LEAST(12, 6)
+using host_numa_memory_pools_supported_t = __dev_attr<::cudaDevAttrHostNumaMemoryPoolsSupported>;
+static constexpr host_numa_memory_pools_supported_t host_numa_memory_pools_supported{};
+#  endif // ^^^ _CCCL_CTK_AT_LEAST(12, 6) ^^^
+
+#  if _CCCL_CTK_AT_LEAST(13, 0)
+using host_memory_pools_supported_t = __dev_attr<::cudaDevAttrHostMemoryPoolsSupported>;
+static constexpr host_memory_pools_supported_t host_memory_pools_supported{};
+#  endif // ^^^ _CCCL_CTK_AT_LEAST(13, 0) ^^^
+
 // Combines major and minor compute capability in a 100 * major + 10 * minor format, allows to query full compute
 // capability in a single query
 struct compute_capability_t
