@@ -25,6 +25,7 @@
 
 #  include <cuda/std/__cstddef/types.h>
 #  include <cuda/std/__exception/cuda_error.h>
+#  include <cuda/std/__host_stdlib/stdexcept>
 #  include <cuda/std/__internal/namespaces.h>
 #  include <cuda/std/__limits/numeric_limits.h>
 #  include <cuda/std/__type_traits/always_false.h>
@@ -35,8 +36,6 @@
 #    include <dlfcn.h>
 #  endif
 
-#  include <stdexcept>
-
 #  include <cuda.h>
 
 #  include <cuda/std/__cccl/prologue.h>
@@ -951,7 +950,7 @@ __cutensormap_size_bytes(::cuda::std::size_t __num_items, ::CUtensorMapDataType
     case ::CU_TENSOR_MAP_DATA_TYPE_FLOAT16:
       if (__num_items > __max_size / 2)
       {
-        _CCCL_THROW(::std::invalid_argument{"Number of items must be less than or equal to 2^64 / 2"});
+        _CCCL_THROW(std::invalid_argument, "Number of items must be less than or equal to 2^64 / 2");
       }
       return __num_items * 2;
     case ::CU_TENSOR_MAP_DATA_TYPE_INT32:
@@ -962,7 +961,7 @@ __cutensormap_size_bytes(::cuda::std::size_t __num_items, ::CUtensorMapDataType
     case ::CU_TENSOR_MAP_DATA_TYPE_TFLOAT32_FTZ:
       if (__num_items > __max_size / 4)
       {
-        _CCCL_THROW(::std::invalid_argument{"Number of items must be less than or equal to 2^64 / 4"});
+        _CCCL_THROW(std::invalid_argument, "Number of items must be less than or equal to 2^64 / 4");
       }
       return __num_items * 4;
     case ::CU_TENSOR_MAP_DATA_TYPE_INT64:
@@ -970,7 +969,7 @@ __cutensormap_size_bytes(::cuda::std::size_t __num_items, ::CUtensorMapDataType
     case ::CU_TENSOR_MAP_DATA_TYPE_FLOAT64:
       if (__num_items > __max_size / 8)
       {
-        _CCCL_THROW(::std::invalid_argument{"Number of items must be less than or equal to 2^64 / 8"});
+        _CCCL_THROW(std::invalid_argument, "Number of items must be less than or equal to 2^64 / 8");
       }
       return __num_items * 8;
 #  if _CCCL_CTK_AT_LEAST(12, 8)
