@@ -512,27 +512,37 @@ _CCCL_CONCEPT __kernel_has_default_config =
  * function should be used instead
  *
  * @tparam Dimensions
- * cuda::hierarchy_dimensions instance that describes dimensions
+ * cuda::hierarchy instance that describes dimensions
  * of thread hierarchy in this configuration object
  *
  * @tparam Options
  * Types of options that were added to this configuration object
  */
-template <typename Dimensions, typename... Options>
+template <typename Hierarchy, typename... Options>
 struct kernel_config
 {
-  Dimensions dims;
-  ::cuda::std::tuple<Options...> options;
+  using hierarchy_type = Hierarchy;
+  using options_type   = ::cuda::std::tuple<Options...>;
 
   static_assert(::cuda::std::_And<::cuda::std::is_base_of<__detail::launch_option, Options>...>::value);
   static_assert(__detail::no_duplicate_options<Options...>);
 
-  constexpr kernel_config(const Dimensions& dims, const Options&... opts)
-      : dims(dims)
-      , options(opts...) {};
-  constexpr kernel_config(const Dimensions& dims, const ::cuda::std::tuple<Options...>& opts)
-      : dims(dims)
-      , options(opts) {};
+  constexpr kernel_config(const Hierarchy& hierarchy, const Options&... opts)
+      : __hierarchy(hierarchy)
+      , __options(opts...) {};
+  constexpr kernel_config(const Hierarchy& hierarchy, const ::cuda::std::tuple<Options...>& opts)
+      : __hierarchy(hierarchy)
+      , __options(opts) {};
+
+  [[nodiscard]] _CCCL_API constexpr const Hierarchy& hierarchy() const noexcept
+  {
+    return __hierarchy;
+  }
+
+  [[nodiscard]] _CCCL_API constexpr const ::cuda::std::tuple<Options...>& options() const noexcept
+  {
+    return __options;
+  }
 
   /**
    * @brief Add a new option to this configuration
@@ -546,8 +556,8 @@ struct kernel_config
   template <typename... NewOptions>
   [[nodiscard]] auto add(const NewOptions&... new_options) const
   {
-    return kernel_config<Dimensions, Options..., NewOptions...>(
-      dims, ::cuda::std::tuple_cat(options, ::cuda::std::make_tuple(new_options...)));
+    return kernel_config<Hierarchy, Options..., NewOptions...>(
+      __hierarchy, ::cuda::std::tuple_cat(__options, ::cuda::std::make_tuple(new_options...)));
   }
 
   /**
@@ -574,8 +584,8 @@ struct kernel_config
     // can't use fully qualified kernel_config name here because of nvcc bug,
     // TODO remove __make_config_from_tuple once fixed
     return __make_config_from_tuple(
-      dims.combine(__other_config.dims),
-      ::cuda::std::tuple_cat(options, ::cuda::std::apply(__filter_options<Options...>{}, __other_config.options)));
+      __hierarchy.combine(__other_config.hierarchy()),
+      ::cuda::std::tuple_cat(__options, ::cuda::std::apply(__filter_options<Options...>{}, __other_config.options())));
   }
 
   /**
@@ -605,6 +615,10 @@ struct kernel_config
       return *this;
     }
   }
+
+private:
+  Hierarchy __hierarchy;
+  ::cuda::std::tuple<Options...> __options;
 };
 
 // We can consider removing the operator&, but its convenient for in-line
@@ -613,19 +627,19 @@ template <typename Dimensions, typename... Options, typename NewLevel>
 _CCCL_HOST_API constexpr auto
 operator&(const kernel_config<Dimensions, Options...>& config, const NewLevel& new_level) noexcept
 {
-  return kernel_config(hierarchy_add_level(config.dims, new_level), config.options);
+  return kernel_config(hierarchy_add_level(config.hierarchy(), new_level), config.options());
 }
 
 template <typename NewLevel, typename Dimensions, typename... Options>
 _CCCL_HOST_API constexpr auto
 operator&(const NewLevel& new_level, const kernel_config<Dimensions, Options...>& config) noexcept
 {
-  return kernel_config(hierarchy_add_level(config.dims, new_level), config.options);
+  return kernel_config(hierarchy_add_level(config.hierarchy(), new_level), config.options());
 }
 
 template <typename L1, typename Dims1, typename L2, typename Dims2>
 _CCCL_HOST_API constexpr auto
-operator&(const level_dimensions<L1, Dims1>& l1, const level_dimensions<L2, Dims2>& l2) noexcept
+operator&(const hierarchy_level_desc<L1, Dims1>& l1, const hierarchy_level_desc<L2, Dims2>& l2) noexcept
 {
   return kernel_config(::cuda::make_hierarchy(l1, l2));
 }
@@ -649,7 +663,7 @@ operator&(const kernel_config<Dimensions, Options...>& config, const Option& opt
 template <typename... Levels,
           typename Option,
           typename = ::cuda::std::enable_if_t<::cuda::std::is_base_of_v<__detail::launch_option, Option>>>
-[[nodiscard]] constexpr auto operator&(const hierarchy_dimensions<Levels...>& dims, const Option& option) noexcept
+[[nodiscard]] constexpr auto operator&(const hierarchy<Levels...>& dims, const Option& option) noexcept
 {
   return kernel_config(dims, option);
 }
@@ -671,10 +685,9 @@ template <typename... Levels,
  * resulting kernel configuration object
  */
 template <typename BottomUnit, typename... Levels, typename... Opts>
-[[nodiscard]] constexpr auto
-make_config(const hierarchy_dimensions<BottomUnit, Levels...>& dims, const Opts&... opts) noexcept
+[[nodiscard]] constexpr auto make_config(const hierarchy<BottomUnit, Levels...>& dims, const Opts&... opts) noexcept
 {
-  return kernel_config<hierarchy_dimensions<BottomUnit, Levels...>, Opts...>(dims, opts...);
+  return kernel_config<hierarchy<BottomUnit, Levels...>, Opts...>(dims, opts...);
 }
 
 /**
@@ -773,7 +786,7 @@ template <typename Dimensions, typename... Options>
 
       return __status;
     },
-    config.options);
+    config.options());
 }
 } // namespace __detail
 
@@ -782,7 +795,7 @@ template <typename Dimensions, typename... Options>
 template <class _Dims, class... _Opts>
 _CCCL_DEVICE_API decltype(auto) dynamic_shared_memory(const kernel_config<_Dims, _Opts...>& __config) noexcept
 {
-  auto& __opt = __detail::find_option_in_tuple<__detail::launch_option_kind::dynamic_shared_memory>(__config.options);
+  auto& __opt = __detail::find_option_in_tuple<__detail::launch_option_kind::dynamic_shared_memory>(__config.options());
   using _Opt  = ::cuda::std::remove_reference_t<decltype(__opt)>;
   static_assert(!::cuda::std::is_same_v<_Opt, __detail::option_not_found>,
                 "Dynamic shared memory option not found in the kernel configuration");
