@@ -1,11 +1,10 @@
-# Copyright (c) 2024-2025, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+# Copyright (c) 2024-2026, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
 #
 #
 # SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 
 from typing import Callable, cast
 
-import numba
 import numpy as np
 
 from .. import _bindings
@@ -17,8 +16,11 @@ from .._cccl_interop import (
     set_cccl_iterator_state,
     to_cccl_value_state,
 )
-from .._utils import protocols
-from .._utils.protocols import get_data_pointer, validate_and_get_stream
+from .._utils.protocols import (
+    get_data_pointer,
+    is_device_array,
+    validate_and_get_stream,
+)
 from .._utils.temp_storage_buffer import TempStorageBuffer
 from ..iterators._iterators import IteratorBase
 from ..op import OpAdapter, OpKind, make_op_adapter
@@ -31,7 +33,7 @@ def get_init_kind(
     match init_value:
         case None:
             return _bindings.InitKind.NO_INIT
-        case _ if isinstance(init_value, DeviceArrayLike):
+        case _ if is_device_array(init_value):
             return _bindings.InitKind.FUTURE_VALUE_INIT
         case _:
             return _bindings.InitKind.VALUE_INIT
@@ -43,7 +45,6 @@ class _Scan:
         "d_in_cccl",
         "d_out_cccl",
         "init_value_cccl",
-        "op",
         "op_cccl",
         "device_scan_fn",
         "init_kind",
@@ -75,9 +76,7 @@ class _Scan:
                 self.init_value_cccl = cccl.to_cccl_input_iter(
                     cast(DeviceArrayLike, init_value)
                 )
-                value_type = numba.from_dtype(
-                    protocols.get_dtype(cast(DeviceArrayLike, init_value))
-                )
+                value_type = get_value_type(cast(DeviceArrayLike, init_value))
                 init_value_type_info = self.init_value_cccl.value_type
 
             case _bindings.InitKind.VALUE_INIT:
@@ -119,6 +118,7 @@ class _Scan:
         temp_storage,
         d_in,
         d_out,
+        op: Callable | OpAdapter,
         num_items: int,
         init_value: np.ndarray | DeviceArrayLike | GpuStruct | None,
         stream=None,
@@ -126,6 +126,10 @@ class _Scan:
         set_cccl_iterator_state(self.d_in_cccl, d_in)
         set_cccl_iterator_state(self.d_out_cccl, d_out)
 
+        # Update op state for stateful ops
+        op_adapter = make_op_adapter(op)
+        op_adapter.update_op_state(self.op_cccl)
+
         match self.init_kind:
             case _bindings.InitKind.FUTURE_VALUE_INIT:
                 # We know that the init_value_cccl is an Iterator, so this cast
@@ -226,9 +230,9 @@ def exclusive_scan(
         stream: CUDA stream for the operation (optional)
     """
     scanner = make_exclusive_scan(d_in, d_out, op, init_value)
-    tmp_storage_bytes = scanner(None, d_in, d_out, num_items, init_value, stream)
+    tmp_storage_bytes = scanner(None, d_in, d_out, op, num_items, init_value, stream)
     tmp_storage = TempStorageBuffer(tmp_storage_bytes, stream)
-    scanner(tmp_storage, d_in, d_out, num_items, init_value, stream)
+    scanner(tmp_storage, d_in, d_out, op, num_items, init_value, stream)
 
 
 # TODO Figure out `sum` without operator and initial value
@@ -293,6 +297,6 @@ def inclusive_scan(
         stream: CUDA stream for the operation (optional)
     """
     scanner = make_inclusive_scan(d_in, d_out, op, init_value)
-    tmp_storage_bytes = scanner(None, d_in, d_out, num_items, init_value, stream)
+    tmp_storage_bytes = scanner(None, d_in, d_out, op, num_items, init_value, stream)
     tmp_storage = TempStorageBuffer(tmp_storage_bytes, stream)
-    scanner(tmp_storage, d_in, d_out, num_items, init_value, stream)
+    scanner(tmp_storage, d_in, d_out, op, num_items, init_value, stream)
