@@ -1,13 +1,11 @@
-# Copyright (c) 2025, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+# Copyright (c) 2025-2026, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
 #
 #
 # SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 
 from typing import Callable
 
-import numba
-
-from .. import _bindings
+from .. import _bindings, types
 from .. import _cccl_interop as cccl
 from .._caching import cache_with_registered_key_functions
 from .._cccl_interop import call_build, set_cccl_iterator_state
@@ -26,8 +24,6 @@ class _ThreeWayPartition:
         "d_second_part_out_cccl",
         "d_unselected_out_cccl",
         "d_num_selected_out_cccl",
-        "select_first_part_op",
-        "select_second_part_op",
         "select_first_part_op_cccl",
         "select_second_part_op_cccl",
     ]
@@ -51,10 +47,10 @@ class _ThreeWayPartition:
         # Compile ops - partition predicates return uint8 (boolean)
         value_type = cccl.get_value_type(d_in)
         self.select_first_part_op_cccl = select_first_part_op.compile(
-            (value_type,), numba.types.uint8
+            (value_type,), types.uint8
         )
         self.select_second_part_op_cccl = select_second_part_op.compile(
-            (value_type,), numba.types.uint8
+            (value_type,), types.uint8
         )
 
         self.build_result = call_build(
@@ -76,6 +72,8 @@ class _ThreeWayPartition:
         d_second_part_out,
         d_unselected_out,
         d_num_selected_out,
+        select_first_part_op: Callable | OpAdapter,
+        select_second_part_op: Callable | OpAdapter,
         num_items: int,
         stream=None,
     ):
@@ -85,6 +83,12 @@ class _ThreeWayPartition:
         set_cccl_iterator_state(self.d_unselected_out_cccl, d_unselected_out)
         set_cccl_iterator_state(self.d_num_selected_out_cccl, d_num_selected_out)
 
+        # Update op state for stateful ops
+        first_op_adapter = make_op_adapter(select_first_part_op)
+        second_op_adapter = make_op_adapter(select_second_part_op)
+        first_op_adapter.update_op_state(self.select_first_part_op_cccl)
+        second_op_adapter.update_op_state(self.select_second_part_op_cccl)
+
         stream_handle = protocols.validate_and_get_stream(stream)
 
         if temp_storage is None:
@@ -195,14 +199,18 @@ def three_way_partition(
         num_items: Number of items to partition
         stream: CUDA stream for the operation (optional)
     """
+    # Create adapters to support stateful ops
+    first_op_adapter = make_op_adapter(select_first_part_op)
+    second_op_adapter = make_op_adapter(select_second_part_op)
+
     partitioner = make_three_way_partition(
         d_in,
         d_first_part_out,
         d_second_part_out,
         d_unselected_out,
         d_num_selected_out,
-        select_first_part_op,
-        select_second_part_op,
+        first_op_adapter,
+        second_op_adapter,
     )
     tmp_storage_bytes = partitioner(
         None,
@@ -211,6 +219,8 @@ def three_way_partition(
         d_second_part_out,
         d_unselected_out,
         d_num_selected_out,
+        first_op_adapter,
+        second_op_adapter,
         num_items,
         stream,
     )
@@ -222,6 +232,8 @@ def three_way_partition(
         d_second_part_out,
         d_unselected_out,
         d_num_selected_out,
+        first_op_adapter,
+        second_op_adapter,
         num_items,
         stream,
     )
