@@ -1,13 +1,11 @@
-# Copyright (c) 2024-2025, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+# Copyright (c) 2024-2026, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
 #
 #
 # SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 
 from typing import Callable
 
-import numba
-
-from .. import _bindings
+from .. import _bindings, types
 from .. import _cccl_interop as cccl
 from .._caching import cache_with_registered_key_functions
 from .._cccl_interop import call_build, set_cccl_iterator_state
@@ -29,7 +27,6 @@ class _UniqueByKey:
         "d_out_keys_cccl",
         "d_out_items_cccl",
         "d_out_num_selected_cccl",
-        "op",
         "op_cccl",
     ]
 
@@ -50,7 +47,7 @@ class _UniqueByKey:
 
         # Compile the op - unique_by_key expects bool return (comparison)
         value_type = cccl.get_value_type(d_in_keys)
-        self.op_cccl = op.compile((value_type, value_type), numba.types.uint8)
+        self.op_cccl = op.compile((value_type, value_type), types.uint8)
 
         self.build_result = call_build(
             _bindings.DeviceUniqueByKeyBuildResult,
@@ -70,6 +67,7 @@ class _UniqueByKey:
         d_out_keys: DeviceArrayLike | IteratorBase,
         d_out_items: DeviceArrayLike | IteratorBase,
         d_out_num_selected: DeviceArrayLike,
+        op: Callable | OpKind | OpAdapter,
         num_items: int,
         stream=None,
     ):
@@ -79,6 +77,10 @@ class _UniqueByKey:
         set_cccl_iterator_state(self.d_out_items_cccl, d_out_items)
         set_cccl_iterator_state(self.d_out_num_selected_cccl, d_out_num_selected)
 
+        # Update op state for stateful ops
+        op_adapter = make_op_adapter(op)
+        op_adapter.update_op_state(self.op_cccl)
+
         stream_handle = validate_and_get_stream(stream)
         if temp_storage is None:
             temp_storage_bytes = 0
@@ -183,6 +185,7 @@ def unique_by_key(
         d_out_keys,
         d_out_items,
         d_out_num_selected,
+        op,
         num_items,
         stream,
     )
@@ -194,6 +197,7 @@ def unique_by_key(
         d_out_keys,
         d_out_items,
         d_out_num_selected,
+        op,
         num_items,
         stream,
     )
