@@ -1,18 +1,14 @@
-# Copyright (c) 2025, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+# Copyright (c) 2025-2026, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+#
 #
 # SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 
 import ctypes
 
-import numba
-from llvmlite import ir  # noqa: F401
-from numba import cuda, types  # noqa: F401
-from numba.core.datamodel.registry import default_manager  # noqa: F401
-from numba.core.extending import as_numba_type, intrinsic  # noqa: F401
+from numba import cuda
 
+from .. import types
 from .._caching import cache_with_registered_key_functions
-from .._cccl_interop import get_dtype
-from ..struct import make_struct_type
 from ._iterators import (
     IteratorBase,
     IteratorKind,
@@ -21,13 +17,32 @@ from ._iterators import (
 
 
 class PermutationIteratorKind(IteratorKind):
-    pass
+    def __init__(
+        self,
+        value_type: types.TypeDescriptor,
+        values_kind: IteratorKind,
+        indices_kind: IteratorKind,
+    ):
+        self.values_kind = values_kind
+        self.indices_kind = indices_kind
+
+    def __repr__(self):
+        return f"PermutationIteratorKind({self.value_type}, {self.values_kind}, {self.indices_kind})"
+
+    def __eq__(self, other):
+        return (
+            type(self) is type(other)
+            and self.values_kind == other.values_kind
+            and self.indices_kind == other.indices_kind
+        )
+
+    def __hash__(self):
+        return hash((type(self), self.values_kind, self.indices_kind))
 
 
 @cache_with_registered_key_functions
 def _generate_advance_and_dereference_methods(values, indices):
-    values_state_type = values.state_type
-    index_type = indices.value_type
+    from .._intrinsics import make_alloca_intrinsic, make_struct_field_ptr_intrinsic
 
     # Check if values supports output operations
     values_is_output_iterator = False
@@ -38,15 +53,17 @@ def _generate_advance_and_dereference_methods(values, indices):
     except AttributeError:
         pass
 
+    alloca_temp_for_index_type = make_alloca_intrinsic(indices.value_type)
+    alloca_temp_for_value_state = make_alloca_intrinsic(values.state_type)
+    get_value_state_field_ptr = make_struct_field_ptr_intrinsic(0)
+    get_index_state_field_ptr = make_struct_field_ptr_intrinsic(1)
+
     # Create a local namespace to avoid polluting globals
     local_ns = {
-        "intrinsic": intrinsic,
-        "types": types,
-        "ir": ir,
-        "default_manager": default_manager,
-        "cuda": cuda,
-        "index_type": index_type,
-        "values_state_type": values_state_type,
+        "alloca_temp_for_index_type": alloca_temp_for_index_type,
+        "alloca_temp_for_value_state": alloca_temp_for_value_state,
+        "get_value_state_field_ptr": get_value_state_field_ptr,
+        "get_index_state_field_ptr": get_index_state_field_ptr,
     }
 
     # JIT compile value advance/dereference methods
@@ -66,60 +83,6 @@ def _generate_advance_and_dereference_methods(values, indices):
         indices.input_dereference, device=True
     )
 
-    # Define intrinsics for accessing struct fields
-    intrinsic_code = """
-@intrinsic
-def get_value_state_field_ptr(context, struct_ptr_type):
-    def codegen(context, builder, sig, args):
-        struct_ptr = args[0]
-        # Use GEP to get pointer to field at index 0 (value_state)
-        field_ptr = builder.gep(
-            struct_ptr,
-            [ir.Constant(ir.IntType(32), 0), ir.Constant(ir.IntType(32), 0)],
-        )
-        return field_ptr
-
-    struct_model = default_manager.lookup(struct_ptr_type.dtype)
-    field_type = struct_model._members[0]
-    return types.CPointer(field_type)(struct_ptr_type), codegen
-
-@intrinsic
-def get_index_state_field_ptr(context, struct_ptr_type):
-    def codegen(context, builder, sig, args):
-        struct_ptr = args[0]
-        # Use GEP to get pointer to field at index 1 (index_state)
-        field_ptr = builder.gep(
-            struct_ptr,
-            [ir.Constant(ir.IntType(32), 0), ir.Constant(ir.IntType(32), 1)],
-        )
-        return field_ptr
-
-    struct_model = default_manager.lookup(struct_ptr_type.dtype)
-    field_type = struct_model._members[1]
-    return types.CPointer(field_type)(struct_ptr_type), codegen
-
-@intrinsic
-def alloca_temp_for_index_type(context):
-    def codegen(context, builder, sig, args):
-        temp_value_type = context.get_value_type(index_type)
-        temp_ptr = builder.alloca(temp_value_type)
-        return temp_ptr
-
-    return types.CPointer(index_type)(), codegen
-
-@intrinsic
-def alloca_temp_for_value_state(context):
-    def codegen(context, builder, sig, args):
-        temp_state_type = context.get_value_type(values_state_type)
-        temp_ptr = builder.alloca(temp_state_type)
-        return temp_ptr
-
-    return types.CPointer(values_state_type)(), codegen
-"""
-
-    # Execute intrinsic definitions in local namespace
-    exec(intrinsic_code, local_ns)
-
     # Define the advance method
     advance_method_code = """
 def permutation_advance(state, distance):
@@ -193,12 +156,12 @@ def make_permutation_iterator(values, indices):
 
     # Convert arrays to iterators if needed
     if hasattr(values, "__cuda_array_interface__"):
-        values = pointer(values, numba.from_dtype(get_dtype(values)))
+        values = pointer(values)
     elif not isinstance(values, IteratorBase):
         raise TypeError("values must be a device array or iterator")
 
     if hasattr(indices, "__cuda_array_interface__"):
-        indices = pointer(indices, numba.from_dtype(get_dtype(indices)))
+        indices = pointer(indices)
     elif not isinstance(indices, IteratorBase):
         raise TypeError("indices must be an iterator or device array")
 
@@ -210,36 +173,37 @@ def make_permutation_iterator(values, indices):
     # The cvalue and state for PermutationIterator are
     # structs composed of the cvalues and states of the
     # value and index iterators.
+    from ..struct import gpu_struct
+
     class PermutationCValueStruct(ctypes.Structure):
         _fields_ = [
             ("value_state", values.cvalue.__class__),
             ("index_state", indices.cvalue.__class__),
         ]
 
-    PermutationState = make_struct_type(
-        "PermutationState",
-        field_names=("value_state", "index_state"),
-        field_types=(values.state_type, indices.state_type),
+    # Create state struct using gpu_struct
+    PermutationState = gpu_struct(
+        {"value_state": values.state_type, "index_state": indices.state_type},
+        name="PermutationState",
     )
+    state_type = PermutationState._type_descriptor
 
     cvalue = PermutationCValueStruct(values.cvalue, indices.cvalue)
-    state_type = as_numba_type(PermutationState)
     value_type = values.value_type
 
     class PermutationIterator(IteratorBase):
-        iterator_kind_type = PermutationIteratorKind
-
         def __init__(self, values_it, indices_it):
             self._values = values_it
             self._indices = indices_it
+            kind = PermutationIteratorKind(
+                value_type, self._values.kind, self._indices.kind
+            )
             super().__init__(
+                kind=kind,
                 cvalue=cvalue,
                 state_type=state_type,
                 value_type=value_type,
             )
-            self._kind = self.__class__.iterator_kind_type(
-                (value_type, values_it.kind, indices_it.kind), state_type
-            )
 
         @property
         def advance(self):
@@ -258,4 +222,11 @@ def make_permutation_iterator(values, indices):
                 )
             return output_dereference_func
 
+        @property
+        def children(self):
+            return (self._values, self._indices)
+
+        def _rebuild_value_type_from_children(self):
+            self.value_type = self._values.value_type
+
     return PermutationIterator(values, indices)
