@@ -1,4 +1,5 @@
-# Copyright (c) 2025, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+# Copyright (c) 2025-2026, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+#
 #
 # SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 
@@ -17,6 +18,7 @@ class _OpAdapter:
     Provides a unified interface for operators, whether they are:
     - Well-known operations (OpKind.PLUS, OpKind.MAXIMUM, etc.)
     - Stateless user-provided callables
+    - Stateful user-provided callables
     """
 
     def compile(self, input_types, output_type=None) -> Op:
@@ -24,14 +26,30 @@ class _OpAdapter:
         Compile this operator to an Op for CCCL interop.
 
         Args:
-            input_types: Tuple of numba types for input arguments
-            output_type: Optional numba type for return value (inferred if None)
+            input_types: Tuple of TypeDescriptors for input arguments
+            output_type: Optional TypeDescriptor for return value (inferred if None)
 
         Returns:
             Compiled Op object for C++ interop
         """
         raise NotImplementedError("Subclasses must implement this method")
 
+    @property
+    def is_stateful(self) -> bool:
+        """Return True if this op has runtime state."""
+        return False
+
+    def update_op_state(self, cccl_op: Op) -> None:
+        """
+        Update the Op's state bytes.
+
+        Args:
+            cccl_op: The compiled CCCL Op to update
+
+        Default implementation is a no-op (for stateless ops).
+        """
+        pass
+
     @property
     def func(self) -> Callable | None:
         """The underlying callable, if any."""
@@ -66,38 +84,6 @@ class _WellKnownOp(_OpAdapter):
         return self._kind
 
 
-class _StatelessOp(_OpAdapter):
-    """Internal wrapper for stateless callables."""
-
-    __slots__ = ["_func", "_cachable"]
-
-    def __init__(self, func: Callable):
-        self._func = func
-        self._cachable = CachableFunction(func)
-
-    def compile(self, input_types, output_type=None) -> Op:
-        from . import _cccl_interop as cccl
-        from .numba_utils import get_inferred_return_type, signature_from_annotations
-
-        # Try to get signature from annotations first
-        try:
-            sig = signature_from_annotations(self._func)
-        except ValueError:
-            # Infer signature from input/output types
-            if output_type is None or (
-                hasattr(output_type, "is_internal") and not output_type.is_internal
-            ):
-                output_type = get_inferred_return_type(self._func, input_types)
-            sig = output_type(*input_types)
-
-        return cccl.to_stateless_cccl_op(self._func, sig)
-
-    @property
-    def func(self) -> Callable:
-        """Access the wrapped callable."""
-        return self._func
-
-
 # Public aliases
 OpAdapter = _OpAdapter
 
@@ -112,6 +98,8 @@ def make_op_adapter(op) -> OpAdapter:
     Returns:
         A value with appropriate subtype of _BaseOp
     """
+    from ._jit import to_jit_op_adapter
+
     # Already an _OpAdapter instance:
     if isinstance(op, _OpAdapter):
         return op
@@ -120,7 +108,8 @@ def make_op_adapter(op) -> OpAdapter:
     if isinstance(op, OpKind):
         return _WellKnownOp(op)
 
-    return _StatelessOp(op)
+    # It's a Python callable
+    return to_jit_op_adapter(op)
 
 
 __all__ = [
@@ -130,16 +119,10 @@ __all__ = [
 ]
 
 
-# ============================================================================
-# Register key functions
-# ============================================================================
-
 cache_with_registered_key_functions.register(
     _WellKnownOp, lambda op: (op._kind.name, op._kind.value)
 )
 
-cache_with_registered_key_functions.register(_StatelessOp, lambda op: op._cachable)
-
 cache_with_registered_key_functions.register(
     OpKind, lambda kind: (kind.name, kind.value)
 )
