@@ -3,12 +3,11 @@
 #
 # SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 
-from typing import Protocol, runtime_checkable
+from typing import Protocol
 
-import numpy as np
+from .struct import _Struct
 
 
-@runtime_checkable
 class DeviceArrayLike(Protocol):
     """
     Objects representing a device array, having a `.__cuda_array_interface__`
@@ -26,12 +25,4 @@ class StreamLike(Protocol):
     def __cuda_stream__(self) -> tuple[int, int]: ...
 
 
-@runtime_checkable
-class GpuStruct(Protocol):
-    """
-    Type of instances of structs created with gpu_struct().
-    """
-
-    _data: np.ndarray
-    __array_interface__: dict
-    dtype: np.dtype
+GpuStruct = _Struct
